package net.sf.r4h;

import java.io.Serializable;

/**
 * Restrictions that support all Hibernate restriction types and introduces {@link Restrictions.Restriction#disabledIfValueIsNull}
 * 
 * Allows restrictions on properties of associated entities without need of explicitly setting association path.
 * 
 * Can be used in environments(GWT,RMI,Web Services clients) without direct Hibernate dependencies.
 * 
 * The class is immutable and thread safe.
 * 
 * @author vitaliy.se@gmail.com
 * 
 * @since 1.0.0
 */
public final class Restrictions implements Serializable {
	private static final long serialVersionUID = -7038123173749667155L;

	/**
	 * Hibernate restriction types
	 * 
	 */
	enum Type {
		LESS, LESS_OR_EQUAL, EQUAL, NOT_EQUAL, GREATER_OR_EQUAL, GREATER, LIKE, ILIKE, IN,NOT_IN, IS_EMPTY, NOT_EMPTY, IS_NULL, NOT_NULL, SIZE_LESS, SIZE_LESS_OR_EQUAL, SIZE_EQUAL, SIZE_GREATER_OR_EQUAL, SIZE_GREATER, PROPERTY_LESS, PROPERTY_LESS_OR_EQUAL, PROPERTY_EQUAL, PROPERTY_GREATER_OR_EQUAL, PROPERTY_GREATER, EXAMPLE;
	}

	/**
	 *
	 * Can be used in environments(GWT,RMI,Web Services clients) without direct Hibernate dependencies.
	 *
	 * Represents particular type of restriction
	 * 
	 * The class is immutable and thread safe.
	 *
	 */
	public static class Restriction implements Serializable {

		/**
		 * Needed for GWT RPC
		 */
		private Restriction() {
			this(null);
		}

		private static final long serialVersionUID = -293845708751983797L;

		private final Type type;

		private boolean disabledIfValueIsNull;

		public boolean isDisabledIfValueIsNull() {
			return disabledIfValueIsNull;
		}

		/**
		 * The restriction will not be used in case the value for restriction is null
		 * 
		 * @return Same instance of {@link Restrictions} with disabledIfValueIsNull set to true
		 */
		public Restriction disabledIfValueIsNull() {
			this.disabledIfValueIsNull = true;
			return this;
		}

		private Restriction(Type type) {
			this.type = type;
		}

		Type getType() {
			return type;
		}
	}

	public static CaseAwareRestriction less() {
		return new CaseAwareRestriction(Type.LESS);
	};

	public static CaseAwareRestriction lessOrEqual() {
		return new CaseAwareRestriction(Type.LESS_OR_EQUAL);
	};

	public static CaseAwareRestriction equal() {
		return new CaseAwareRestriction(Type.EQUAL);
	}

	public static CaseAwareRestriction notEqual() {
		return new CaseAwareRestriction(Type.NOT_EQUAL);
	}

	public static CaseAwareRestriction greaterOrEqual() {
		return new CaseAwareRestriction(Type.GREATER_OR_EQUAL);
	}

	public static CaseAwareRestriction greater() {
		return new CaseAwareRestriction(Type.GREATER);
	}

	public static CaseAwareRestriction like() {
		return new CaseAwareRestriction(Type.LIKE);
	}

	public static Restriction ilike() {
		return new Restriction(Type.ILIKE);
	}

	public static EmptinessAwareRestriction in() {
		return new EmptinessAwareRestriction(Type.IN);
	}
	
	public static EmptinessAwareRestriction notIn() {
		return new EmptinessAwareRestriction(Type.NOT_IN);
	}

	public static Restriction isEmpty() {
		return new Restriction(Type.IS_EMPTY);
	}

	public static Restriction notEmpty() {
		return new Restriction(Type.NOT_EMPTY);
	}

	public static Restriction isNull() {
		return new Restriction(Type.IS_NULL);
	}

	public static Restriction notNull() {
		return new Restriction(Type.NOT_NULL);
	}

	public static Restriction sizeLess() {
		return new Restriction(Type.SIZE_LESS);
	}

	public static Restriction sizeLessOrEqual() {
		return new Restriction(Type.SIZE_LESS_OR_EQUAL);
	}

	public static Restriction sizeEqual() {
		return new Restriction(Type.SIZE_EQUAL);
	}

	public static Restriction sizeGreaterOrEqual() {
		return new Restriction(Type.SIZE_GREATER_OR_EQUAL);
	}

	public static Restriction sizeGreater() {
		return new Restriction(Type.SIZE_GREATER);
	}

	public static Restriction propertyLess() {
		return new Restriction(Type.PROPERTY_LESS);
	}

	public static Restriction propertyLessOrEqual() {
		return new Restriction(Type.PROPERTY_LESS_OR_EQUAL);
	}

	public static Restriction propertyEqual() {
		return new Restriction(Type.PROPERTY_EQUAL);
	}

	public static Restriction propertyGreaterOrEqual() {
		return new Restriction(Type.PROPERTY_GREATER_OR_EQUAL);
	}

	public static Restriction propertyGreater() {
		return new Restriction(Type.PROPERTY_GREATER);
	}

	public static ExampleRestriction example() {
		return new ExampleRestriction();
	}

	/**
	 * Case sensitivity aware restriction.
	 */
	public static class CaseAwareRestriction extends Restriction {
		private static final long serialVersionUID = 5556311249131241169L;
		private boolean caseSensitive = true;
		
		/**
		 * Needed for GWT RPC
		 */
		private CaseAwareRestriction() {
			this(null);
		}

		private CaseAwareRestriction(Type type) {
			super(type);
		}

		public CaseAwareRestriction caseInsensitive() {
			caseSensitive = false;
			return this;
		}

		public boolean isCaseSensitive() {
			return caseSensitive;
		}

		@Override
		public CaseAwareRestriction disabledIfValueIsNull() {
			return (CaseAwareRestriction) super.disabledIfValueIsNull();
		}

	}

	/**
	 * 
	 * This restriction allows to disable IN statement in case it is empty;
	 */
	public static class EmptinessAwareRestriction extends Restriction {
		private static final long serialVersionUID = 4091181099917296199L;
		private boolean disabledIfEmpty = false;

		public EmptinessAwareRestriction(Type type) {
			super(type);
			if (type != Type.IN && type !=Type.NOT_IN){
				throw new IllegalArgumentException("Only IN and NOT_IN restrictions are supported");
			}
		}

		public EmptinessAwareRestriction disableIfEmpty() {
			disabledIfEmpty = true;
			return this;
		}

		public boolean isDisabledIfValueIsEmpty() {
			return disabledIfEmpty;
		}

		@Override
		public EmptinessAwareRestriction disabledIfValueIsNull() {
			return (EmptinessAwareRestriction) super.disabledIfValueIsNull();
		}

	}

	/**
	 * Restriction for Hibernate example criteria.
	 */
	public static class ExampleRestriction extends Restriction {
		private static final long serialVersionUID = -6029638181078542427L;
		private boolean isLikeEnabled;
		private boolean areZeroesExcluded;
		private boolean isCaseIgnored;
		private boolean isExcludeNone;

		private ExampleRestriction() {
			super(Type.EXAMPLE);
		}

		public ExampleRestriction ignoreCase() {
			isCaseIgnored = true;
			return this;
		}

		public ExampleRestriction enableLike() {
			isLikeEnabled = true;
			return this;
		}

		public ExampleRestriction excludeZeroes() {
			areZeroesExcluded = true;
			return this;
		}

		public ExampleRestriction excludeNone() {
			isExcludeNone = true;
			return this;
		}

		public boolean isLikeEnabled() {
			return isLikeEnabled;
		}

		public boolean areZeroesExcluded() {
			return areZeroesExcluded;
		}

		public boolean isCaseIgnored() {
			return isCaseIgnored;
		}

		public boolean isExcludeNone() {
			return isExcludeNone;
		}

		@Override
		public ExampleRestriction disabledIfValueIsNull() {
			return (ExampleRestriction) super.disabledIfValueIsNull();
		}
	}

}
